////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omJoystickDriver_
#define _omJoystickDriver_

#include <openmaia/commons.h>
#include <openmaia/omObject.h>

#include <wx/wx.h>
#include <wx/joystick.h>



/** @ingroup omdrivers
    OpenMAIA Joystick driver.

    This class is used as a driver for joystick events. <br>
    The position of the joystick cloche is rapresented by two integer values.
    X axis position is rapresented by a value that is 0 when the joystick is to the 
    far left and is set to max when the joystick is to the far right. <br>
    Y axis position is rapresented by a value that is 0 when the joystick is in the
    forward position and is set to max when the joystick is in the backward position.<br>
    In the constructor of the class you specify the percentage of the axis that is
    considerated as left, as right, as up and as down. <br><br>

    This class generates events to object specified in the constructor.<br>
    - Action event is generated when fire button is pressed.
    Using EnableAllEvents() method you can specify to generate Action event when
    one of button 1,2 or 3 is pressed.
    - GoLeft event is generated when X position has a smaller value than the left threshold
    calculated dinamically using the left percentage.
    - GoRight event is generated when X position has a bigger value than the right threshold
    calculated dinamically using the right percentage.
    - GoUp event is generated when Y position has a smaller value than the up threshold
    calculated dinamically using the up percentage.
    - GoDown event is generated when Y position has a bigger value than the down threshold
    calculated dinamically using the down percentage.
*/

class omJoystickDriver: public omObject, wxTimer
{
 private:
  // run parameters
  int tickTime; ///< Pool time.
  int sxThreshold, ///< Runtime threshold.
    dxThreshold, ///< Runtime threshold.
    upThreshold, ///< Runtime threshold.
    downThreshold;  ///< Runtime threshold.
  int maxX,         ///< Maximum X value.
    minX,           ///< Minimum X value.
    maxY,           ///< Maximum Y value.
    minY;           ///< Minimum Y value.
  omObject *toInform ;   ///< Object to inform.
  bool actionOnPress;  ///< Flag.

  float rightPercentage,   ///< Threshold right percentage.
    leftPercentage,        ///< Threshold left percentage.
    upPercentage,          ///< Threshold up percentage.
    downPercentage;        ///< Threshold down percentage.

  // operation variables
  wxJoystick *joy ;   ///< joy object
  bool joystickConnected; ///< not(ErrorState)
  bool isSx, ///< Flag.
    isDx, ///< Flag.
    isUp, ///< Flag.
    isDown, ///< Flag.
    isButton, ///< Flag.
    allButtonsEnabled; ///< Flag.
  int buttonMask;  ///< Button mask for GetButtonState()

  // methods
  void Notify() ;    ///< Timer notify
  void RecalcThresholds();

 public:

  omJoystickDriver(omObject *, int, int, float, float, float, float, bool);
  bool GetError() ;  // return error state
  void Start() ;   // start the event catching
  void Stop() ;   // stop the event catching
  void EnableAllButtons();    // send event on any button pression
  void DisableAllButtons();   // send event on button1 pression (default)
  void SetMaxX(int);
  void SetMinX(int);
  void SetMaxY(int);
  void SetMinY(int);
  
};


#endif // _omJoystickDriver_
