////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omKeyboardDriver_
#define _omKeyboardDriver_

#include <openmaia/commons.h>
#include <openmaia/omcommon.h>
#include <openmaia/omObject.h>
#include <openmaia/omFunctions.h>

#include <wx/wx.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/termios.h>
#include <fcntl.h>


/** @ingroup omdrivers
    OpenMAIA Keyboard Driver.

    This class is a driver used to interface a standard keyboard with the
    OpenMAIA system.<br>
    Some keys of a standard keyboard can be used to generate events for 
    OpenMAIA Applications.<br>
    This driver can handle the status of 5 keys and generate OpenMAIA events to
    an object according with these keys status.<br>
    The code of the keys used to generate events is passed via the constructor
    of the class according with OpenMAIA Key codes avaiable in the
    <a href="omemukeys_8h.html">omemukeys.h</a> header file.
*/
class omKeyboardDriver: public omObject, wxTimer
{
 private:

  long actionKey,    ///< OpenMAIA ACTION key code.
    upKey,           ///< OpenMAIA UP key code.
    downKey,         ///< OpenMAIA DOWN key code.,
    leftKey,         ///< OpenMAIA LEFT key code.
    rightKey;        ///< OpenMAIA RIGHT key code.
  bool grabKeys,     ///< Flag.
    actionOnPress;   ///< Flag.


  bool busy;

  void *systemDependentData ;   ///< System dependent data structure
                                ///< defined in x11 or windows folder.
  long maiaKeyCode[MAXMAIACODES]  ;   ///< Keycodes hash table.
  int tickTime ;  ///< Timer period.

  void Notify() ;   // overrides timer notify
  void SystemDependentConstructor() ;   ///< System dependent part of constructor see in x11 or windows folder.
  void SystemDependentNotify() ;   ///< System dependent notify method.

  void SDFlushKey(long, bool * );   ///< System dependent flush key method.
  // system dependent keyboard functions
  void SDGrabActionKeys() ;  ///< System dependent GrabActionKey method.
  void SDUngrabActionKeys() ;  ///< System dependent UngrabActionKey method.
  void SDFlushKeysState();   ///< System dependent FlushKeysState method.
  void SDDestructor();      ///< System dependent Destructor


 public:
  omObject *toInform ;             ///< Object to send event to.
  omObject *executorController;    ///< Object that controls the OpenMAIA executor class.
  omKeyboardDriver(omObject *, omObject *, 
		   int,  long, long, long, long, long, 
		   bool, bool) ;
  ~omKeyboardDriver();
  void SetOnPress(bool) ;   // set action on press flag
  void SetGrabKeys(bool);   // set grab keys flag
  void Start() ;   // start the event catching
  void Stop() ;   // stop the event catching
  void GrabActionKeys() ;
  void UngrabActionKeys() ;
  void FlushKeysState();  // flush the keys state
};


#endif // _omKeyboardDriver_
