////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#ifndef _omUIButton_
#define _omUIButton_

#include <openmaia/commons.h>
#include <iostream>
#include <vector>
#include <string>
#include <wx/wx.h>
#include <wx/wave.h>   //deprecated since 2.6
//#include <wx/sound.h>

#include <openmaia/omObject.h>
#include <openmaia/omUIApplication.h>


/** @ingroup omui

    OpenMAIA User Interface Button.

    This class is a super class for an OpenMAIA User interface virtual button.
    The class descends from wxFrame and implements a window with no borders and with 
    stay on top feature.
    The stay on top feature works correctly under windows but have some problems with
    GNU/Linux GTK 1.X and doesn't work with GNU/Linux GTK 2.X.
    With GNU/Linux stay on top works correctly with WindowMaker and with GNOME 2.4 (metacity),
    some problems occours with KDE. In particular KDE freezes sometimes.<br>
    The events handled by this object are listed above:

    - onClick: event generated when the user click on the button.
    - onScratch: event generated when the user scratches the mouse pointer over the button.
    - onFocus: event generated when the focus is grabbed by this object.

    The object can also handle images and sounds.

    @todo Implement onScratch method
*/

class omUIButton: public wxFrame, public omObject {
 private:
  // ------------------------------------ PRIVATE FUNCTIONS ------------------------

  // --------- Event Handlers
  void OnMouseButtonUp(wxMouseEvent &event) ;
  void OnPaint(wxPaintEvent &event) ;             
  void OnSetFocus(wxFocusEvent &event) ;

  // ----------- System dependent functions ---
  void SDPutOnTop() ;    ///< System dependent PutOnTop method
  bool SDIsYourWindow( void *) ;  ///< System dependent check window handler.
  void PlaySoundSD(wxString, bool);  ///< System dependent PlaySound method

  // ------------------------------------ PRIVATE VARIABLES ----------------------------

  bool isOnTop ;  ///< onTop flag.

  std::vector<wxString> image_paths ;  ///< Vector of paths of images to render.
  std::vector<wxBitmap *> images ; ///< Vector of images to render.
  std::vector<wxString> sounds ; /// Vector of sounds to render.

  int nsounds;     ///< Number of stored sounds.
  int  nimages ;   ///< Number of stored images.

  // declare event table to catch events
  DECLARE_EVENT_TABLE()

protected:
  // ------------------------------  PROTECTED VARIABLES AND FUNCTIONS --------------------

  int curImage ;   ///< Current image displayed.

  int AddSound( wxString);
  int AddImage(wxString);
  void PlaySound(int );
  void RenderImage(int);

  // virtual functions used to handle events on the buttons
  virtual void OnClick();
  virtual void OnScratch();
  virtual void OnFocus() ;

 public:
  // --------------------------------- PUBLIC FUNCTIONS -------------------------------------

  int x;  ///< x position.
  int y;  ///< y position.
  int w;  ///< width.
  int h;  ///< height.

  //Pointers needed for manual navigation
  omObject *upper;   ///< Upper button (joystick UP).
  omObject *lower;   ///< Lower button (joystick DOWN).
  omObject *left;    ///< Left button (joystick LEFT).
  omObject *right;   ///< Right button (joystick RIGHT).


  // The constructor.
  omUIButton( bool, wxWindow *, int, int, int, int) ;
  // The wx destructor.
  virtual bool Destroy() ;

  // public methods
  void Show();
  void Hide();
  void PutOnTop();

  bool IsYourWindow( void *) ;  // query for window handler

  // -------- public virtual functions
  virtual void PaintDefaultPicture() ;
  virtual void LightOn() ;
  virtual void LightOnSingle();
  virtual void LightOff() ;
  virtual void Execute() ;

} ;

#endif //_omUIButton_
