////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#ifndef _omUITreeNode_
#define _omUITreeNode_

#include <openmaia/commons.h>

#include <iostream>
#include <vector>


#include <openmaia/omObject.h>
#include <openmaia/omUIApplication.h>
#include <openmaia/omUIOptions.h>
#include <openmaia/omUIButton.h>




/** @ingroup omui

    This class is a superclass for a generic node of a maia keyboard tree.
    This class can be a node or a leaf of the tree. It handles some signals passed
    through the tree. These signals are used to handle events for implementing the 
    automatic timed scan of the virtual keyboard.
    The class contains a pointer to a MaiaLittleWindow object. If the node is a leaf
    this pointer points to the leaf object else it's null. You can use this pointer to handle
    your own button inherit from MaiaLittleWindow.
    The class also contains a pointer to GenericNodeData class than contains all data
    needed to the node. You can inherit a your own data class from GenericNodeData.
*/

class omUITreeNode: public omObject {
 protected:
  // ------------------------------------------------- protected variables ---
  omUITreeNode *parent ;   // pointer of parent node
  std::vector<omUITreeNode *> nodes ;    // children of the Node
  omUIButton *leaf ;   // pointer to leaf object if it is a leaf node.
  int nnodes ;    // number of children nodes.
  int currentNode;   // current child node selected in automatic scan mode.
                     // -1 by default on start.
  int scanCount;     // count of loops, used when this branch is selected in automatic scan,
                     // when max tick count is reached the node activity will be released.
  int scanMax;       // max number of scan loops before exit from the branch.
  omObject *data ;    // data associated with the node
  bool isLeaf ;   // true if this is a leaf node.
  



 public:
  bool actioned ;  // true if the button is previously actioned 


  // ------------------------------------------------ public methods ---------------  
  omUITreeNode( omUITreeNode *, omObject *, int ) ;  // Constructor of the class as a node
  omUITreeNode( omUITreeNode *, omUIButton *);  // Constructor of the class as a leaf
  ~omUITreeNode() ;

  void DeleteChildren() ;  // delete all children (used in the root node only)
  void AddNode( omUITreeNode *) ; // Add a new child node 
  void AddLeaf( omUIButton *) ; // Add a new lead node 
  omUITreeNode *GetParentNode() ; // return the parent node pointer 
  bool IsRoot();                  // return true if the node is root
  bool IsYourWindow( void *) ; // check if window is one of leafs
  void DeAction(bool) ;  // Deaction the object ... called by a child
  void Tick() ;  // Pass the tick signal to the node
  void Action() ; // Pass the action signal to the node
  void PaintWindows() ;  // paint default picture of all windows
  void Show() ;   // show all windows of the node
  bool GetActionStatus() ;
  void Repaint() ;
  

  // --- virtual functions
  // Run some commands when the block is selected by the user
  virtual void Execute() ;
  // Highlight the group
  virtual void LightOn() ;
  // Turn the highlight off to the group
  virtual void LightOff() ;
} ;

#endif // _omUITreeNode_

