///////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omExecutor.h>

#include <iostream>

// Catch timer event table
BEGIN_EVENT_TABLE( omExecutor, wxEvtHandler)
  EVT_TIMER(MOUSE_TIMER_ID, omExecutor::OnMouseTimer)
END_EVENT_TABLE()


/** Construct the class by initializating character emulation table 
*/
omExecutor::omExecutor()
  :mouseTimer(this, MOUSE_TIMER_ID)
{
  int i ;   // generic counter

  // initialize mouse variables
  isMouseInMovement = false ;
  for (i=0 ; i<MAXMOUSEBUTTONS; i++)
    isMouseButtonPressed[i] = false ;

  // The character emulation table is system dependent
  this->SDConstructor() ;
}

/** Start all active timers */
void omExecutor::Start()
{
  if ( isMouseInMovement)
    mouseTimer.Start(MOUSE_MOVEMENT_TIMER) ;
}

/** Stop all timers */
void omExecutor::Stop()
{
  mouseTimer.Stop() ;
}


/** Emulate the pressure of char keys to construct the passed
    string using system dependent API
    @param s string to write
*/
void omExecutor::RenderString( wxString s)
{
  int i ;
  for ( i=0; i<s.size(); i++)
    this->SDRenderChar(s.GetChar(i)) ;

}

/** Emulate pressure of a special key.
    A key has a specific maia code that is translated into windows or x11 code
    in funcion of the system where the application is compiled
    @param code maia code of the char, see this manual for a complete list
*/
void omExecutor::PressKey(long code)
{
  this->SDPressKey(code) ;
}

/** Emulate pressure of a special key.
    A key has a specific maia code that is translated into windows or x11 code
    in funcion of the system where the application is compiled
    @param code maia code of the char, see this manual for a complete list
*/
void omExecutor::ReleaseKey( long code)
{
  this->SDReleaseKey(code) ;
}


/** Emulate the mouse movement.
    This functions parses the string and then call The MoveMouse(dx,dy) function
    @param action to implement, may be "MoveMouseLeft", "MoveMouseRight", "MoveMouseUp"
      or "MoveMouseDown"
    @param step movement step in pixel

    @return true if the action stops the mouse, false otherwhise
*/
bool omExecutor::MoveMouse( wxString action, wxString step)
{
  int s ;  // integer step
  long slong;
  bool noValue = false ;

  int dx = 0 ;
  int dy = 0 ;


  if ( step == wxString(_T("")) )   // no value passed
    noValue = true ;
  else    // convert to integer the parameter
    {
      step.ToLong(&slong);
      s=(int)slong;
    }

  // --- mouse UP --
  if ( action == wxString(_T("movemouseup")))
    {
      if (noValue)
	dy = 0 ;
      else
	dy = -s ;
    }

  // --- mouse DOWN --
  if ( action == wxString(_T("movemousedown")))
    {
      if (noValue)
	dy = 0 ;
      else
	dy = s ;
    }

  // --- mouse LEFT --
  if ( action == wxString(_T("movemouseleft")))
    {
      if (noValue)
	dx = 0 ;
      else
	dx = -s ;
    }

  // --- mouse RIGHT --
  if ( action == wxString(_T("movemouseright")))
    {
      if (noValue)
	dx = 0 ;
      else
	dx = s ;
    }

  // call the right function
  return this->MoveMouse(dx,dy) ;

}

/** Emulate the mouse movement.
    This function is two state function, the second time you call it with the same parameter
    the mouse stop its movement.
    You can either call the function with 0 or without step to stop the mouse movement.
    @param dx x increment
    @param dy y increment

    @return true if the action stops the mouse, false otherwhise
*/
bool omExecutor::MoveMouse( int dx, int dy)
{
  // check if the command is at second shot
  if ( dx == dxMouseMovement )   // stop the mouse
    dxMouseMovement = 0 ;
  else
    dxMouseMovement = dx ;

  if ( dy == dyMouseMovement )   // stop the mouse
    dyMouseMovement = 0 ;
  else
    dyMouseMovement = dy ;

  if ( isMouseInMovement)
    {
      if ( dx==0 || dy == 0)    // stop the timer
	{
	  mouseTimer.Stop() ;
	  isMouseInMovement = false ;
	}
    }
  else
    {
      mouseTimer.Start(MOUSE_MOVEMENT_TIMER) ;
      isMouseInMovement = true ;
    }
  return !isMouseInMovement ;
}


/** Emulate a click on a mouse button.
    @param but wich button emulate, may be LEFTBUTTON, CENTRALBUTTON or RIGHTBUTTON
*/
void omExecutor::ClickMouse(int but)
{
  this->SDPressMouseButton(but) ;
  this->SDReleaseMouseButton(but) ;
  this->SDReleaseMouseButton(but) ;
  //this->SDPressMouseButton(but) ;
  //this->SDReleaseMouseButton(but) ;
}

/** Emulate mouse drag/drop.
    The first time the command is lounched it emulate mouse pression and the second
    time emulate button release.
    The function handles the status of automatic scan of the associated keyboard
    if it isn't NULL.
    @param but wich button emulate, may be LEFTBUTTON, CENTRALBUTTON or RIGHTBUTTON 
*/
void omExecutor::DragOrDropMouse( int but )
{
  if (isMouseButtonPressed[but])     // release button and restart automatic scan
    {
      this->SDReleaseMouseButton( but) ;
      isMouseButtonPressed[but] = false ;
    }
  else
    {
      this->SDPressMouseButton(but) ;
      isMouseButtonPressed[but] = true ;
    }
}



long omExecutor::ExecuteApplication(wxString command){
  return(::wxExecute(command));
}



// ----------------------------------------------- private functions
void omExecutor::OnMouseTimer(wxTimerEvent &event)
{
  this->SDMoveMouse() ;
}

