////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omJoystickDriver.h>



/** Initialize the class.<br>
    By default Action event is generated by fire button of the joystick.

    @param object Object to inform if event occoured.
    @param tick Pooling period in milliseconds.
    @param jn Joystick Number, maybe wxJOYSTICK1 or wxJOYSTICK2.
    @param lp Left percentage, usually 2/10.
    @param rp Right percentage, usually 8/10.
    @param up Up percentage, usually 2/10.
    @param dp Down percentage, usually 8/10.
    @param onpress Flag that specify if the action must be taken on the
                   pression or on the releas of the button key
*/
omJoystickDriver::omJoystickDriver(omObject *object, int tick, int jn,
				   float lp, float rp, float up, float dp,
				   bool onpress)
{
  // store objects
  toInform = object ;
  tickTime = tick;
  actionOnPress = onpress;

  allButtonsEnabled = false;
  buttonMask = wxJOY_BUTTON1;
  joystickConnected = false ;

  // connect to joystick
  joy = new wxJoystick(jn);
  if (joy->IsOk())
    joystickConnected = true ;

  // set default value for axis estremes.
  maxX = 0;
  minX = 0;
  maxY = 0;
  minY = 0;
  leftPercentage = lp;
  rightPercentage = rp;
  upPercentage = up;
  downPercentage = dp;


  if (joystickConnected)    // calculate thresholds
    {
      // by default read these values from the joystick.
      // In some cases joystick returns incorrect values under linux.
      // In these cases user can specify correct values using SetMaxX, SetMaxY, SetMinX, SetMinY 
      // methods.
      int minX = joy->GetXMin() ;
      int maxX = joy->GetXMax() ;
      int minY = joy->GetYMin() ;
      int maxY = joy->GetYMax() ;

      int xinterval = (int) ((maxX-minX)) ;
      int yinterval = (int) ((maxY-minY)) ;
      sxThreshold = (int) ((maxX-minX) * lp) ;
      dxThreshold = (int) ((maxX-minX) * rp) ;
      upThreshold = (int) ((maxY-minY) * up) ;
      downThreshold = (int) ((maxY-minY) * dp) ;

//       std::cout << "x:" << minJoyX << 
// 	"\nX:" << maxJoyX <<
// 	"\ny:" << minJoyY <<
// 	"\nY:" << maxJoyY <<
// 	"\nsxT:" << sxThreshold <<
// 	"\ndxT:" << dxThreshold <<
// 	"\nupT:" << upThreshold <<
// 	"\ndownT:" << downThreshold << std::endl;

    }

}

/** Return error state.<br>
    If the function return true means that joystick was not found and don't work correctly

    @return Error state.
*/
bool omJoystickDriver::GetError()
{
  return !joystickConnected;
}

/** Start pooling joystick status */
void omJoystickDriver::Start()
{
  wxTimer::Start(tickTime);
}

/** Stop pooling joystick status */
void omJoystickDriver::Stop()
{
  wxTimer::Stop() ;
}

/** Enable Action event sending when one of button 1,2 or 3 is pressed. */
void omJoystickDriver::EnableAllButtons()
{
  allButtonsEnabled = true ;
  buttonMask = wxJOY_BUTTON1 | wxJOY_BUTTON2 | wxJOY_BUTTON3 | wxJOY_BUTTON4;
}

/** Disable Action event sending when one of button 1,2 or 3 is pressed.<br>
    Action event is sent only on button 1 pression.
 */
void omJoystickDriver::DisableAllButtons()
{
  allButtonsEnabled = false ;
  buttonMask = wxJOY_BUTTON1;
}

/** Set new Maximum X value. Thresholds will be automatically recalculed.
    @param value New value.
*/
void omJoystickDriver::SetMaxX(int value)
{
  maxX = value;
  this->RecalcThresholds();
}

/** Set new Minimum X value. Thresholds will be automatically recalculed.
    @param value New value.
*/
void omJoystickDriver::SetMinX(int value)
{
  minX = value;
  this->RecalcThresholds();
}

/** Set new Maximum Y value. Thresholds will be automatically recalculed.
    @param value New value.
*/
void omJoystickDriver::SetMaxY(int value)
{
  maxY = value;
  this->RecalcThresholds();
}

/** Set new Minimum Y value. Thresholds will be automatically recalculed.
    @param value New value.
*/
void omJoystickDriver::SetMinY(int value)
{
  minY = value;
  this->RecalcThresholds();
}



// private methods ----------------------------------------------------------------------------------


// timer notify
// check joy status
void omJoystickDriver::Notify()
{
  if (joystickConnected)
    {
      wxPoint pt=joy->GetPosition() ;

      int xpos=pt.x;
      int ypos=pt.y ;

      int pino = joy->GetButtonState();
      //      std::cout << "posizione:" << xpos << "," << ypos << " - " << pino << std::endl;

      // check if any joystick command is given
      if (isSx) {
	if (xpos>sxThreshold) {
	  if (!actionOnPress)   // acion on release
	    toInform->GoLeft() ;
	  isSx = false ;     // release semaphore
	}
      }else
	if (xpos<sxThreshold) {
	  if (actionOnPress)   // action on press
	  toInform->GoLeft() ;
	  isSx=true ;
	}
      if (isDx) {
	if (xpos<dxThreshold){
	  if (!actionOnPress)
	    toInform->GoRight() ;
	  isDx = false ;
	}
      }else
	if (xpos>dxThreshold) {
	  if (actionOnPress)
	    toInform->GoRight() ;
	  isDx=true ;
	}

      if (isUp) {
	if (ypos>upThreshold) {
	  if (!actionOnPress)
	    toInform->GoUp() ;
	  isUp=false ;
	  
	}
      } else
	if (ypos<upThreshold){
	  if (actionOnPress)
	    toInform->GoUp() ;
	  isUp=true ;
	}

      if (isDown) {
	if (ypos<downThreshold) {
	  if (!actionOnPress)
	    toInform->GoDown() ;
	  isDown=false;
	}
      }else
	if (ypos>downThreshold) {
	  if (actionOnPress)
	    toInform->GoDown() ;
	  isDown=true ;
	}

      if (isButton) {
	if (!( joy->GetButtonState() & wxJOY_BUTTON1)) {
	  if (!actionOnPress)
	    toInform->Action() ;
	  isButton = false ;
	}
      } else
	if (joy->GetButtonState() & wxJOY_BUTTON1) {
	  if (actionOnPress)
	    toInform->Action() ;
	  isButton=true ;
	}
    }

}


/** Recalc thresholds after changing values of axixs extremes */
void omJoystickDriver::RecalcThresholds()
{
  int xinterval = (int) ((maxX-minX)) ;
  int yinterval = (int) ((maxY-minY)) ;
  sxThreshold = (int) ((maxX-minX) * leftPercentage) ;
  dxThreshold = (int) ((maxX-minX) * rightPercentage) ;
  upThreshold = (int) ((maxY-minY) * upPercentage) ;
  downThreshold = (int) ((maxY-minY) * downPercentage) ;
}
