////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omSerialDriver.h>



/** Initialize the class 
    @param object object to inform if event occoured
    @param tick timer pooling interval in milliseconds
    @param sport serial port in un*x format, for example COM1 is /dev/ttyS0 ecc...
    @param onpress actionOnPress flag
*/
omSerialDriver::omSerialDriver(omObject *object, int tick, wxString sport, bool onpress)
{
  toInform = object ;
  tickTime = tick ;
  serialPort = sport ;

  // set private vars value
  errorStatus=false ;
  RIActivated = false;
  CTSActivated=false;
  DSRActivated=false;
  CDActivated=false;

  int i;

  // set all buttons to action event
  for (i=0; i<OMSERIALBUTTONNUMBER; i++)
    fptr[i] = &omObject::Action;

  // enable all buttons to generate events
  for (i=0; i<OMSERIALBUTTONNUMBER; i++)
    enableButton[i] = true;

  // open serial port
  this->OpenSerialPort() ;
}

/** Set OnPress flag
    @param op value of the onpress flag
*/
void omSerialDriver::SetOnPress(bool op)
{
  actionOnPress = op ;
}


/** Start the event notify with the time interval specified in the constructor*/
void omSerialDriver::Start()
{
  wxTimer::Start(tickTime) ;
}

/** Start the event notify with the time interval specified in the constructor*/
void omSerialDriver::Stop()
{
  wxTimer::Stop() ;
}

/** Assign Action event to a button.
    @param button button number.
*/
void omSerialDriver::AssignActionEvent(int button)
{
  fptr[button] = &omObject::Action;
  enableButton[button] = true ;
}

/** Assign GoUp event to a button.
    @param button button number.
*/
void omSerialDriver::AssignUpEvent(int button)
{
  fptr[button] = &omObject::GoUp;
  enableButton[button] = true ;
}


/** Assign GoDown event to a button.
    @param button button number.
*/
void omSerialDriver::AssignDownEvent(int button)
{
  fptr[button] = &omObject::GoDown;
  enableButton[button] = true ;
}

/** Assign GoLeft event to a button.
    @param button button number.
*/
void omSerialDriver::AssignLeftEvent(int button)
{
  fptr[button] = &omObject::GoLeft;
  enableButton[button] = true ;
}

/** Assign GoRight event to a button.
    @param button button number.
*/
void omSerialDriver::AssignRightEvent(int button)
{
  fptr[button] = &omObject::GoRight;
  enableButton[button] = true ;
}

/** Disable event generation from all buttons.
    Event generation is abilitated when Assign<EVENT> method is called.

    @param button button number.
*/
void omSerialDriver::DisableButtonEvent(int button) 
{
  enableButton[button] = false ;
}

/** Return error status.<br>
    Error may be true if the class encountred problems in opening serial port.
    @return Error status.
*/
bool omSerialDriver::GetError()
{
  return errorStatus;
}

// private methods -----------------------------------------------------------------------
void omSerialDriver::OpenSerialPort()
{
  int control = 0 ;
  serialFd = open((char *)serialPort.c_str(),O_RDWR ) ;

  if ( serialFd == -1) {
#ifdef MAIA_DEBUG
    std::cout << "omSerialDriver::OpenSerialPort - Error opening serial port" << std::endl;
#endif
    errorStatus=true;
    //exit(-1) ;
    return ;
  }

  // set RTS up for interface alimentation
  control |= TIOCM_RTS;
  ioctl(serialFd, TIOCMSET, &control) ;
}


void omSerialDriver::Notify()
{
  int status =0;

  if (errorStatus)
    return ;

  ioctl(serialFd, TIOCMGET, &status) ;

  if (actionOnPress)
    {
      if((status & TIOCM_RI) && !RIActivated)
	{
	  RIActivated = true ;
	  (toInform->*fptr[0])() ;
	}

      if((status & TIOCM_CTS) && !CTSActivated)
	{
	  CTSActivated = true ;
	  (toInform->*fptr[1])() ;
	}

      if((status & TIOCM_DSR) && !DSRActivated)
	{
	  DSRActivated = true ;
	  (toInform->*fptr[2])() ;
	}

      if((status & TIOCM_CD) && !CDActivated)
	{
	  CDActivated = true ;
	  (toInform->*fptr[3])() ;
	}


      // button 0 released
      if(!(status & TIOCM_RI) && RIActivated)
	{
	  RIActivated = false ;
	}

      // button 1 released
      if(!(status & TIOCM_CTS) && CTSActivated)
	{
	  CTSActivated = false ;
	}

      // button 2 released
      if(!(status & TIOCM_DSR) && DSRActivated)
	{
	  DSRActivated = false ;
	}

      // button 3 released
      if(!(status & TIOCM_CD) && CDActivated)
	{
	  CDActivated = false ;
	}


    }
  else
    {
      if((status & TIOCM_RI) && !RIActivated)
	RIActivated = true ;

      if((status & TIOCM_CTS) && !CTSActivated)
	CTSActivated = true ;

      if((status & TIOCM_DSR) && !DSRActivated)
	DSRActivated = true ;

      if((status & TIOCM_CD) && !CDActivated)
	CDActivated = true ;

      // button 0 released
      if(!(status & TIOCM_RI) && RIActivated)
	{
	  (toInform->*fptr[0])() ;
	  RIActivated = false ;
	}

      // button 1 released
      if(!(status & TIOCM_CTS) && CTSActivated)
	{
	  //std::cout << "ok" << std::endl ;
	  (toInform->*fptr[1])() ;
	  CTSActivated = false ;
	}

      // button 2 released
      if(!(status & TIOCM_DSR) && DSRActivated)
	{
	  //std::cout << "ok" << std::endl ;
	  (toInform->*fptr[2])() ;
	  DSRActivated = false ;
	}

      // button 3 released
      if(!(status & TIOCM_CD) && CDActivated)
	{
	  //std::cout << "ok" << std::endl ;
	  (toInform->*fptr[3])() ;
	  CDActivated = false ;
	}

    }

}
