////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


//#include <openmaia/config.h>



#include <openmaia/omcommon.h>

#include <openmaia/omUIButton.h>

#include <wx/image.h>

#ifdef WINDOWS_SYSTEM
#define NOMINMAX
#include <windows.h>
#include <wx/msw/winundef.h>
#endif


// define events for the omUIButton class

//  EVT_PAINT( omUIButton::OnPaint)
BEGIN_EVENT_TABLE( omUIButton, wxFrame) 
  EVT_LEFT_UP( omUIButton::OnMouseButtonUp)
  EVT_PAINT( omUIButton::OnPaint)
  EVT_SET_FOCUS( omUIButton::OnSetFocus)
END_EVENT_TABLE()

// private functions ----------

/** Left mouse click internal handler. */
void omUIButton::OnMouseButtonUp( wxMouseEvent &event) 
{
  //call the dummy virtual function
  OnClick() ;
}

/** Redraw the current bitmap on OnPain event. */
void omUIButton::OnPaint( wxPaintEvent &event)
{
  if ( curImage != -1) {    // if -1 no bitmap was paint
    wxPaintDC dc(this) ;    // use wxPaintDC to handle the OnPain event and wxClientDC 
                            // to repaint by the user
    dc.DrawBitmap( *images[curImage], 0, 0, TRUE) ;
  }

}

/** OnFocus internal event handler. */
void omUIButton::OnSetFocus( wxFocusEvent &event)
{
  this->OnFocus() ;
}

//  ---------------------------------- PUBLIC FUNCTIONS -----------------------

/** The constructor.
    Build the window and put it 'alwais on top' if onTop is true.
    @param onTop Put on top flag
    @param parent Pointer of the parent window
    @param xp x position 
    @param yp y position
    @param wp width
    @param hp height
 */
omUIButton::omUIButton( bool onTop, wxWindow *parent, int xp, int yp,
				    int wp, int hp)
  //   : wxFrame( parent, -1, wxString(_T("OpenMaia")), wxPoint(xp,yp), wxSize(hp,wp), wxSIMPLE_BORDER | wxFRAME_FLOAT_ON_PARENT| wxFRAME_NO_TASKBAR )
  : wxFrame( parent, -1, wxString(_T("OpenMaia")), wxPoint(xp,yp), wxSize(wp,hp), wxSIMPLE_BORDER | wxFRAME_NO_TASKBAR )
{
  isOnTop = onTop ;
  curImage = -1 ;   // no current image

  // initialize internal variables
  x = xp ;
  y = yp ; 
  w = wp ;
  h = hp ;

  nsounds = 0 ;
  nimages = 0 ;

  // add the image handlers used by wxImage to load disk pictures
  // you can load BMP, JPG, PNG, GIF, TIF
//   wxImage::AddHandler( new wxBMPHandler ) ; 
//   wxImage::AddHandler( new wxJPEGHandler) ; 
//   wxImage::AddHandler( new wxPNGHandler) ; 
//   wxImage::AddHandler( new wxGIFHandler) ; 
//   wxImage::AddHandler( new wxTIFFHandler) ; 


}

/** wxWindows descrutor of the class.
    Deallocates all allocated objects.
*/
bool omUIButton::Destroy()
{
  int i ;
  // delete all images
  for (i=0; i<nimages; i++) {
//    wxBitmap *im = images[i] ;
//    delete im ;
    delete(images[i]);
  }
  // call parent's destroy
  wxFrame::Destroy() ;
}


// ------------------------------------------------------------ GENERIC FUNCTIONS -----

/** Add a sound path to button and return the index of the added sound.
    It's not convenient to store all the sound in memory because it can be very large so 
    we store only the path of the sound file and we play them when required. 
    @param path Path of the sound file
    @return Index of sound added.
*/
int omUIButton::AddSound( wxString path)
{
  // allocate a buffer to store the string
  sounds.insert(sounds.end(), path) ;
  nsounds++ ;

  return(nsounds-1) ;
}

/** Add an image path to panel and returns the index of the inserted image.
    Load the bitmap in a wxBitmap object and store the wxBitmap obj in the button "memory". 
    @param path of the image file, can be jpg, tiff, bmp or png
    @return Index of the added image.
*/
int omUIButton::AddImage( wxString path )
{
  // we store either the path and the wxBitmpap to direct draw the image without reload

  nimages++ ;
  // store the path
  image_paths.insert(image_paths.end(), path) ;

  // we use wxImage to load generic format, resize and than we convert to wxBitmap
  // to direct draw via wxClientDC

  // load the image in a wxImage object
  wxImage *bufImg = new wxImage() ;
  bufImg->LoadFile( wxString(_T(path.c_str())) ) ;
  bufImg->Rescale(w,h) ;    // scale the image to window button dimensions

  // convert to wxBitmap
  //wxBitmap *bufBmp = new wxBitmap(bufImg->ConvertToBitmap()) ;  //deprecated since 2.6
  wxBitmap *bufBmp = new wxBitmap(bufImg) ;

  delete(bufImg) ;  // delete temporary objs before quitting

  // store the wxBitmap
  images.insert(images.end(), bufBmp) ;

  return(nimages-1) ;
}

/** Play the sound stored in the index position.
    @param index index of the sound to play
*/
void omUIButton::PlaySound( int index)
{
  /// @todo In linux must debug wxWindows wave handling.
#ifdef LINUX_SYSTEM
  return;
#endif

  ///////////////////////////////////////////////////////////////////
  // wxWindows sound class doesn't hanle correctly subfolders
  ///////////////////////////////////////////////////////////////////

   wxString buf ;   // will store the sound to play
   buf = sounds[index] ;

// #ifdef MAIA_DEBUG
//   std::cout << "omUIButton::PlaySound -> sound file: " << buf << std::endl;
// #endif

//   wxWave *s = new wxWave( wxString(_T(buf.c_str()))) ;
//   s->Play(FALSE,FALSE) ;    // play in asyncronous  mode

  //////////////////////////////////////////////////////////////////


// #ifdef WINDOWS_SYSTEM
//   wxString toPlay = omFunctions::GetWindowsString(buf);
//   ::PlaySound( toPlay.c_str(), NULL, SND_ASYNC | SND_FILENAME) ;
// #endif

  this->PlaySoundSD(buf,true);

  //delete s;
}

/** Render the image stored in the index position.
    @param index of the image to render
*/
void omUIButton::RenderImage( int index )
{
  wxClientDC dc(this) ;      // use wxClientDC to render the image
                             // wxPaintDC is used only to handle OnPaint event

  dc.DrawBitmap( *images[index], 0, 0, TRUE) ;

  curImage = index ;

}


/** Show the window and put 'on top' if required.
    This is necessary under linux because X11 needs GTK window handler. 
*/
void omUIButton::Show()
{
  wxFrame::Show() ;
  if (isOnTop)
    this->PutOnTop() ;
  // get the currentbutton

  wxGetApp().Yield();
}

/** Hide the window without destroy it. */
void omUIButton::Hide()
{
  if (this->IsShown()) {
    wxFrame::Show(false) ;
    wxGetApp().Yield() ;
  }
}

/** Put the window on the top of other windows */
void omUIButton::PutOnTop()
{
  // call the system dependent method
  this->SDPutOnTop() ;
}

/** Use System specific Api to determine if the passed window handler belongs to
    this window.
    @param wh system specific window handler to check
*/
bool omUIButton::IsYourWindow(void *wh)
{
  return this->SDIsYourWindow(wh) ;
}

// -------------------------------------------------- DUMMY VIRTUAL FUNCTIONS ------

// ---protected:
/** Called when the button is clicked.
    This is a virtual function and must be overriden to handle the signal.
 */
void omUIButton::OnClick() {
  // do nothing
}

/** Called when the button is scratched. 
    This is a virtual function and must be overriden to handle the signal.
*/
void omUIButton::OnScratch() {
  // do nothing
}
 
// ---public:

/** Pain the default picture of the window.
    This is a virtual function and must be implemented by the child */
void omUIButton::PaintDefaultPicture()
{
  // Virtual function
}

/** Called to turn on the highlighted mode of the window in automatic scan mode.
    This is a virtual function and must be overriden to handle the signal.
*/
void omUIButton::LightOn() {
  // do nothing
}

/** Called to turn on the highlighted mode of the window in automatic scan mode.
    This signal is generated when the button is scanned alone and is not part
    of a group of bottons.
    This is a virtual function and must be overriden to handle the signal.
*/
void omUIButton::LightOnSingle() {
  // do nothing
}


/** Called to turn off the highlighted mode of the in automatic scan mode.
    This is a virtual function and must be overriden to handle the signal.
*/
void omUIButton::LightOff() {
  // do nothing
}

/** Called when the button is focused */
void omUIButton::OnFocus()
{
  // do nothing
}

void omUIButton::Execute() {
  std::cout << "esecuzione comando" << std::endl ;
}
