////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <openmaia/omUITreeNode.h>

#include <openmaia/omcommon.h>

/** Constructor of the class.The constructor of the class generates a new object and allocates 
    data.
    omUITreeNode derived class and GenericNodeData derived class can be used
    to handle specific node and specific data.
    @param p Pointer to the parent node
    @param d Pointer to the Node Data
    @param ms Maximum number of scan cycles before exit in automatic scan mode. \\ 
              (Set to -1 to infinite loop)
*/
omUITreeNode::omUITreeNode( omUITreeNode *p, omObject *d, int ms)
{
  // initialize data pointers
  data = data ;
  parent = p      ;
  

  // initialize all variables to default value
  currentNode = -1 ;
  nnodes = 0 ;
  scanCount = 0;
  isLeaf = false ;    // false by default
  leaf = NULL ;
  actioned = false ;

  scanMax = ms ;

}


/** Constructor of the class as a tree lead.The constructor of the class generates a new object 
    and allocates data.
    omUITreeNode derived class and GenericNodeData derived class can be used
    to handle specific node and specific data.
    @param p Pointer to the parent node
    @param but Pointer to button (a omUIButton derived class)
*/
omUITreeNode::omUITreeNode( omUITreeNode *p, omUIButton *but)
{
  // initialize defaults really don't care :))
  currentNode = -1 ;
  nnodes = 0 ;
  scanCount = 0;
  actioned = false ;
  scanMax = 0 ;


  leaf = but ;
  isLeaf = true ;
  parent = p ;
}

/* Deallocate the object */
omUITreeNode::~omUITreeNode()
{
  if (leaf != NULL) {
    leaf->Hide() ;
    //delete leaf ;
    leaf->Close() ;
    //wxGetApp().Yield() ;
    //delete leaf ;
    //leaf->Destroy();
    wxGetApp().Yield() ;
  }

  // delete all children
  int i ;
  for (i=0; i<nnodes; i++)
    delete nodes[i] ;
  
}


/** Delete all children wihout deleting this node.
    used in the root node only.
*/
void omUITreeNode::DeleteChildren()
{
  int i ;
  for (i=0; i<nnodes; i++)
    delete nodes[i] ;
}


/** Add a new node as a child of this node in the tree structure.
    @param node Pointer to node to add.
*/
void omUITreeNode::AddNode( omUITreeNode *node) 
{
  nodes.insert(nodes.end(), node) ;  // insert node in the dinamic vector
  nnodes++ ;
}

/** Add a new node as a leaf of this node in the tree structure.
    @param l Pointer to omUIButton to insert as leaf.
*/
void omUITreeNode::AddLeaf( omUIButton *l) 
{
  // create the leaf node to insert
  omUITreeNode *node = new omUITreeNode( this, l) ;

  // insert node in the dinamic vector
  nodes.insert(nodes.end(), node) ;  
  nnodes++ ;
}


/** Return the parent node pointer
    @return parent node pointer
*/
omUITreeNode *omUITreeNode::GetParentNode()
{
  return parent ;
}


/** Query the node for root node.
    @return true if node is root, false otherwhise
*/
bool omUITreeNode::IsRoot()
{
  if (parent==NULL)
    return true;
  else
    return false;
}


/** Check if the passed window handler belong to the associated leaf
    or to any child leaf.
    The handler is a void * so this method is not system dependent.
    @param wh window handler to check

    @return true if wh is handler of leaf or any child leaf, false otehrwise
*/
bool omUITreeNode::IsYourWindow( void *wh)
{
  if ( isLeaf)   // check if the leaf associated has wh window handler
    {
      return leaf->IsYourWindow(wh) ;
    }
  else  // test children
    {
      int i ;
      for( i=0; i < nnodes; i++) 
	if ( nodes[i]->IsYourWindow(wh))
	  return true ;   // find wh in child
    }

  // wh is not one of the children
  return false ;
}


/* Deaction the node.
   @param l true if the child is a leaf
*/
void omUITreeNode::DeAction(bool l)
{
  // retrive application options
  omUIOptions *options = (omUIOptions *) wxGetApp().objects.options ;

  int scanResumeFrom = RESUMEFROMTHIS ;  // default
  bool delayAfterClick = false ;  // default
  bool delayAfterAction = true ;  // default
  bool exitBlockAfterClick = false ;   // default
  bool scanReverseDirection = false  ;   // defaults values


  if ( options != NULL) {
    scanResumeFrom = options->scanResumeFrom ;
    delayAfterClick = options->delayAfterClick ;
    delayAfterAction = options->delayAfterAction ;
    exitBlockAfterClick = options->exitBlockAfterClick ;
    scanReverseDirection = options->scanReverseDirection ;
  }

  actioned = false ;

  // if the node is a leaf exit from the function
  if (l) {
    if ( exitBlockAfterClick)
      {
	this->currentNode = -1 ;
	parent->DeAction(false) ;
      }

    if ( !delayAfterAction)
      {
	// sent tick message to the keyboard
	omObject *keyb = wxGetApp().objects.keyboard ;
	keyb->Tick() ;
      }    
    return ;
  }


  switch(scanResumeFrom)
    {
    case RESUMEFROMFIRST:
      if (!scanReverseDirection)
	currentNode = -1 ;
      else
	currentNode = nnodes ;

      break ;

    case RESUMEFROMTHIS:
      if (!scanReverseDirection)
	currentNode-- ;
      break ;

    case RESUMEFROMNEXT:
      break ;
    }

}


/** Pass the tick signal to the node. Tick signal is used to comunicate to the node
    that a scan event was occoured.
    If the node is not a leaf it comunicates the signal to children. If the node is a
    leaf the signal is used to change display status (scanned or not scanned).
*/
void omUITreeNode::Tick()
{
  bool delayAfterEndBlock = false ;   // default

  omUIOptions *options = (omUIOptions *) wxGetApp().objects.options ;

  if ( options != NULL) {
    delayAfterEndBlock = options->delayAfterEndBlock ;
  }


  // when a node is actioned the tick signal is passed to the current child.
  if (actioned) {   // pass the tick signal to the selected node
    nodes[currentNode]->Tick() ;
    return ;
  } 

  if ( scanMax == -1 )
    scanCount = 0 ;   // infinite loop
  
  // check if scan count reached last node before deaction the block
  if ( (scanCount == (scanMax-1)) && (currentNode == nnodes-1)) {
    parent->DeAction(false) ;    // deaction the parent node, light off current node
    this->LightOff() ;      // and reset local counters
    this->scanCount = 0 ;
    this->currentNode = -1 ;

    if (!delayAfterEndBlock) {
      // tick the keyboard
      (wxGetApp().objects.keyboard)->Tick() ; 
    }

    return ;  
  }

  
  // when a node in not actioned the tick signal causes it to change the children mode
  // to action next child in the tree
  int nextNode = currentNode + 1 ;
  if ( nextNode >= nnodes ) {
    nextNode = 0 ;   // return to the first node
    scanCount++ ;    // increment counter of scan cycles
  }


  // light off the current node
  if ( currentNode >= 0 )
    nodes[currentNode]->LightOff() ;
  // light on the next node
  nodes[nextNode]->LightOn() ;

  currentNode = nextNode ;   // change the pointer to the node
  
}

/** Pass the action signal to the node. Action indicate that user fired the button or the
    group of buttons.
*/
void omUITreeNode::Action() 
{
  if (currentNode == -1)
    return ;

  // if the node is actioned it passes the sigal to the current selected node
  if (actioned) { 
    nodes[currentNode]->Action() ;
    return ;
  }

  // if the node is not actioned it passes the Execute command to the current node and
  // freezes itself
  this->actioned = true ;
  nodes[currentNode]->LightOff() ;
  nodes[currentNode]->Execute() ;

}

/** Paint default picture of all windows */
void omUITreeNode::PaintWindows()
{
  if (isLeaf)
    {
      leaf->PaintDefaultPicture() ;
      return ;
    }

  // else pass the signal to all children
  int i;
  for( i=0; i<nnodes; i++)
    nodes[i]->PaintWindows() ;
}

/** Show all the windows of this node */
void omUITreeNode::Show()
{
  if (isLeaf)   // show the leaf and return
    {
      leaf->Show() ;  
      return ;
    }

  // else pass the show signal to all children
  int i;
  for( i=0; i<nnodes; i++)
    nodes[i]->Show() ;
}


// --------------------------------------------------------------- VIRTUAL FUNCTIONS ---

/** Force the node to light on state.
    This method is called to highlight the group.
    The function by default passes the signal to all the children and if it is a leaf
    the signal is passed to the leaf object. This function can be overridden to do specific
    work.
*/  
void omUITreeNode::LightOn() {
  // if the node is a leaf pass the lighton signal to the leaf object and return
  // if parent is not actioned pass the LightOn Signal
  if (isLeaf && !(parent->GetActionStatus() || parent->IsRoot())) 
    {
      leaf->LightOn() ;
      return;
    }
  

  // if parent is actioned pass the LightOnSingle Signal used to indicate
  // that the leaf is lighted alone
  if (isLeaf && (parent->GetActionStatus() || parent->IsRoot())) 
    {
      leaf->LightOnSingle() ;
      return ;
    }

  // else pass the lighton signal to all the children
  for (int i=0; i<nnodes; i++)
    nodes[i]->LightOn() ;

}

/** Force the node to light off state
    This method is called turn off the highlight status of the group.
    The function by default passes the signal to all the children and if it is a leaf
    the signal is passed to the leaf object. This function can be overridden to do specific
    work.
*/  
void omUITreeNode::LightOff() {
  // if the node is a leaf pass the lighton signal to the leaf object and return
  if (isLeaf) {
    leaf->LightOff() ;
    return ;
  }

  // else pass the lighton signal to all the children
  for (int i=0; i<nnodes; i++)
    nodes[i]->LightOff() ;

}



/** Run application dependent functions when the block is selected by the user.
    The Execute command is called by the parent node when it is actioned by the user.
    By default if the node is a leaf the leaf Execute() method is called.
    The function can be overriddent to handle specific cases.
*/
void omUITreeNode::Execute() 
{
  bool delayAfterClick = false ;   // default

  omUIOptions *options = (omUIOptions *) wxGetApp().objects.options ;

  if ( options != NULL) {
    delayAfterClick = options->delayAfterClick ;
  }


  if (isLeaf) {
    leaf->Execute() ;
    parent->DeAction(true) ;
  }
  else {
    if ( !delayAfterClick) {
      // tick the keyboard 
      (wxGetApp().objects.keyboard)->Tick() ;
    }
  }
}

/** Return the status of the actioned flag.
    @return actioned flag status.
*/
bool omUITreeNode::GetActionStatus()
{
  if (parent != NULL)
    return parent->actioned;
  else
    return false;
}



/** Send Repain() message to all subnodes.
    This method is used to repaint all tree or some parts of it.
*/
void omUITreeNode::Repaint()
{
  // if leaf paint default picture
  if (isLeaf)
    {
      leaf->PaintDefaultPicture();
      return ;
    }

  // send message to subnodes
  int i;
  for (i=0; i<nnodes; i++)
    nodes[i]->Repaint();
}
