////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////


#include <openmaia/omXMLParser.h>

// -------------------------------------------------------------------
// C functions used by the XML parser
// ------------------------------------------------------------------

// called by the parser at the beggining of the document
// calls the appropriate class method
void startDocument(void *user_data)
{
  omXMLParser *keyClass = (omXMLParser *) user_data ;
  keyClass->AtStartDocument() ;
}

// called by the parser at the end of the document
// calls the appropriate class method
void endDocument(void *user_data)
{
  omXMLParser *keyClass = (omXMLParser *) user_data ;
  keyClass->AtEndDocument() ;
}


// called by the parser at the beginning of an element
// fills a MaiaParameters object and calls the right class method
void startElement(void *user_data, const xmlChar *name, const xmlChar **attrs)
{
  omXMLParser *keyClass = (omXMLParser *) user_data ;
  
  omXMLParameters p ;
    
  int idx = 0 ;
  if (attrs != NULL ) {
    while ( attrs[idx] != NULL ) {//each attribute element will be added.
      wxString parname(_T(attrs[idx])) ;
      wxString parvalue(_T(attrs[idx+1])) ;
      p.AddParameter( parname, parvalue) ;
      idx += 2 ;
    }
  }

  keyClass->AtStartTag( wxString(_T(name)), p) ;

}

// called by the parser at the end of each element. It calls the right class method.
void endElement(void *user_data, const xmlChar *name)
{
  omXMLParser *keyClass = (omXMLParser *) user_data ;
  
  keyClass->AtEndTag(wxString(_T(name))) ;
}

// C function that calls the libxml2 parser
// return 0 if the document is ok, -1 if is not well formed.
int saxParseFile(xmlSAXHandlerPtr sax, void *user_data, const char *filename) {
    int ret = 0;
    xmlParserCtxtPtr ctxt;

    ctxt = xmlCreateFileParserCtxt(filename);
    if (ctxt == NULL) return -1;
    ctxt->sax = sax;
    ctxt->userData = user_data;

    xmlParseDocument(ctxt);

    if (ctxt->wellFormed)
        ret = 0;
    else
        ret = -1;
    if (sax != NULL)
        ctxt->sax = NULL;
    xmlFreeParserCtxt(ctxt);
    
    return ret;
}


/** Construct the class by inizializating the XML parser.
    This class need gnome libxml2 library and headers.
*/
omXMLParser::omXMLParser()
{

  // C structure containing informations about C functions used by the parser.

  static xmlSAXHandler localOMSAXHandler = {
    0, /* internalSubset */
    0, /* isStandalone */
    0, /* hasInternalSubset */
    0, /* hasExternalSubset */
    0, /* resolveEntity */
    0, /* getEntity */
    0, /* entityDecl */
    0, /* notationDecl */
    0, /* attributeDecl */
    0, /* elementDecl */
    0, /* unparsedEntityDecl */
    0, /* setDocumentLocator */
    (startDocumentSAXFunc)startDocument, /* startDocument */
    (endDocumentSAXFunc)endDocument, /* endDocument */
    (startElementSAXFunc)startElement, /* startElement */
    (endElementSAXFunc)endElement, /* endElement */
    0, /* reference */
    0, /* characters */
    0, /* ignorableWhitespace */
    0, /* processingInstruction */
    0, /* comment */
    0, /* warning */
    0, /* error */
    0, /* fatalError */
  };

  // store the handler in a class variable
  omSAXHandler = &localOMSAXHandler ;
   
}

/** Parse XML File.
    The function starts the XML parser and call internal functions to handle parsing operations.
    @param f name of the xml file to load

    @return true if keyboard is correcly loaded, false if not
*/
bool omXMLParser::Parse( wxString f)
{
  //wxString fileToParse = _T("\"") + f + _T("\"");
  wxString pathToParse = f;

  // get path from filename
  int lastSlash = pathToParse.Find('/', TRUE);
  wxString fileToParse = pathToParse;
  wxString filePath;
  bool dirChanged = false;

  wxString currentPath = wxGetCwd();
  if (lastSlash != -1) 
    {
      dirChanged = true;
      filePath = pathToParse.Left(lastSlash);
      fileToParse = pathToParse.Right( pathToParse.Len()-lastSlash-1);
      // change to file path (because under windows sax parser doesn't handle
      // path with space.
      chdir( filePath.c_str());
    }


#ifdef WINDOWS_SYSTEM
  //fileToParse = omFunctions::GetWindowsString(fileToParse);
#endif

  int r = saxParseFile(omSAXHandler, this, (const char*)fileToParse.c_str()) ;

  // if directory has changed come back to original folder
  if (dirChanged)
    chdir(currentPath.c_str());

  if ( r == 0 )
    return true ;
  else
    return false ;
}

/** This method is called by the parser when a start of document is found.
    This is a virtual method and must be overridden to implement specific functions.
    The rootNode private variable must be initialized here.
    This function is public because is called externally by the C functions of the parser.
    The user must not call this function by hand.
*/
void omXMLParser::AtStartDocument()
{
  // do nothing.
}

/** This method is called by the parser when an end of document is found.
    This is a virtual method and must be overridden to implement specific functions.
    This function is public because is called externally by the C functions of the parser.
    The user must not call this function by hand.
*/
void omXMLParser::AtEndDocument()
{
  // do nothing.
}

/** This method is called by the parser when a tag open event is found.
    This is a virtual method and must be overridden to implement specific functions.
    This function is public because is called externally by the C functions of the parser.
    The user must not call this function by hand.

    @param n name of the tag
    @param p pars of the tag.
*/
void omXMLParser::AtStartTag(wxString n, omXMLParameters p)
{
  // do nothing.
}

/** This method is called by the parser when a tag close event is found.
    This is a virtual method and must be overridden to implement specific functions.
    This function is public because is called externally by the C functions of the parser.
    The user must not call this function by hand.

    @param n name of the tag
*/
void omXMLParser::AtEndTag(wxString n)
{
  // do nothing.
}


/** Get a standard header to insert in the configuration files
    automatically generated by the system.
*/
wxString omXMLParser::GetXMLOptionsHeader()
{
  wxString header = _T("<!-- This file is automagically generated by OpenMAIA Version ") ;
  header <<  _T(VERSION) << _T("-->\n\n") ;
  return header;
}
