///////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include <unistd.h>


#include <iostream>

#include "maiaButton.h" 


// needed to pass events to the parents
BEGIN_EVENT_TABLE(maiaButton,omUIButton)

END_EVENT_TABLE() 


/** The constructor. Initialize the MaiaButton adding images, sounds and other data.
    @param *parent Window that contain this MaiaButton
    @param OnTop If true put MaiaButton alwais on top.
    @param data MaiaButtonData structure containing the data of the button
*/
maiaButton::maiaButton(wxWindow *parent, bool OnTop, maiaButtonData *data)
  :omUIButton(OnTop,parent,data->x,data->y,data->w,data->h)
{
  isCurrent = false ;


  //NULL all pointer for joystick navigation
  this->upper=NULL;
  this->lower=NULL;
  this->left=NULL;
  this->right=NULL;

  // store the data in the current object 
  buttonData = data ;

  // set default parameters
  haveScanSound = false ;
  haveSelSound = false ;

  // insert the tree images in the structure 
  normalIndex = this->AddImage(data->normalImage) ;
  selIndex = this->AddImage(data->selImage) ;
  scanIndex = this->AddImage(data->scanImage) ;

  // insert the sounds 
  if ( data->scanSound != _T("") ) {   //have scan sound
    scanSoundIndex = this->AddSound(data->scanSound) ;
    haveScanSound = true ;
  }

  if ( data->selSound != _T("")) {   // have sel sound
    selSoundIndex = this->AddSound(data->selSound) ;
    haveSelSound = true ;
  }

}

/** Show the button and set the cursor on it if the button is
    selected by scan mode
*/
void maiaButton::Show()
{
  omUIButton::Show() ;
  if (isCurrent) {
    this->Scan() ;
    wxGetApp().Yield() ;
  }
  
}

/** Clean button from any attributes. Paint 'normal' image. */
void maiaButton::Clean()
{
  if (isCurrent) 
    this->Scan();
  else
    this->RenderImage(normalIndex) ;
  
}

/** Set Select attribute to button. Paint 'selected' image. */
void maiaButton::Select()
{
  this->RenderImage(selIndex) ;
}

/** Set Scanned attribute to button. Paint 'scanned' image. */
void maiaButton::Scan()
{
  this->RenderImage(scanIndex); 
}


/** Mouse click event.
    Implement MaiaLittleWindow virtual function.
    Automatically called when the button is clicked.
*/
void maiaButton::OnClick(){
  this->RenderPushButton() ;
  if (haveSelSound) 
    {
      this->PlaySound(selSoundIndex);
    }
  this->SendSimulation() ;
} 

/** Call the Clean() method to paint the default picture of the window */
void maiaButton::PaintDefaultPicture()
{
  this->Clean() ;
}


/** Implement the LightOn method that is invoked by the automatic scan system
    when the button is automatically scanned.
    The function render the scan image on the button.
*/
void maiaButton::LightOn()
{
  this->Scan() ;
}

/** Implement the LightOnsingle method that is invoked by the automatic scan system
    when the button is automatically scanned.
    Check for soundEnabled option end play the sound if needed. 
    The function render the scan image on the button.
*/
void maiaButton::LightOnSingle()
{
  if (haveScanSound) {
    this->PlaySound(scanSoundIndex);
  }
  this->Scan() ;
}


/** Implement the LightOff method that is invoked by the automatic scan system
    to turn of the scan mark.
    The function render the normal image on the button.
*/
void maiaButton::LightOff()
{
  this->Clean() ;
}

/** Implement the Execute method that is invoked by the automatic scan system
    to execute the code associated with the button.
    The method insert the execute code in the MaiaEventQueue structure of the application.
*/
void maiaButton::Execute()
{
  //this->RenderPushButton() ;
  if (haveSelSound) 
    {
      this->PlaySound(selSoundIndex);
    }
  this->SendSimulation() ;
}


// ------------------------------- private methods

/** This functions simulates the pressure of a button by rendering select image
   and after few milliseconds rendering last rendered image
*/
void maiaButton::RenderPushButton()
{
  int ci = curImage ; // index of current Image
  this->Select() ;  // render selected image
  this->Refresh() ;
  this->Update() ;
  wxGetApp().Yield() ;  // flush events now
  wxUsleep(PUSHBUTTONTIMEINTERVAL) ;  // wait a bit
  this->RenderImage(ci) ; // render last rendered image
  this->Refresh() ;
  this->Update() ;
  wxGetApp().Yield() ; 
}

/** This function send simulation to the event queue */
void maiaButton::SendSimulation()
{
  // retrive informations about serve connector object
  omConnector *connector = (omConnector *) wxGetApp().objects.connector ;

  // reset the counter
  buttonData->ResetActionFetchCounter() ;
  // fetch all actions
  while ( buttonData->MoreActions()) {
    omCommand *a = buttonData->GetNextAction() ;
    int i;
    // build TCP message
    omTCPMessage mess;   // message to send
    mess.command = a->GetCommand();
    for (i=0; i < a->GetParCount(); i++)
      mess.AddParameter( a->GetParameter(i));

    // retrive queue from application and send data to it
    omUIActionQueue *q = (omUIActionQueue *) wxGetApp().objects.actionsQueue ;
    if ( q != NULL )
      q->Put(*a) ;

    // send the command to the connector if exists
    if ( connector != NULL)
      connector->SendMessage(mess);

  }
  
}


/** Handle on focus event.
    When a button receive focus the focus is retured to the window that 
    lost the focus.
*/
void maiaButton::OnFocus()
{
  omUIGrantFocus2User *focuser = (omUIGrantFocus2User *)
               wxGetApp().objects.focusChecker ;
  if ( focuser != NULL )
    focuser->SetLastFocusedWindow() ;
}

/** Set current flag */
void maiaButton::SetCurrent()
{
  isCurrent = true ;
}

/** Reset current flag */
void maiaButton::ResetCurrent() 
{
  isCurrent = false ;
}
