////////////////////////////////////////////////////////////////////////////////
// This code is part of the MaiaProject free software
//
// Keyboard and Mouse Emulator - Virtual keyboard Engine
//
// Copyright (c) 2002-2003 Luca Clivio <luca.clivio@maiaproject.org>
// Tel. +39-347-2538040
// via B.Croce 2, 20037 Paderno Dugnano (Milano) - Italy
//
// Developers:
//    Simone Mangano <simone.mangano@maiaproject.org>
//    Andrea Tosato  <andrea.tosato@maiaproject.org>
//
//  -----------------------------------------------------------------------------
//  This program is free software; This program is free software; 
//  you can redistribute it and/or modify it under the terms of the 
//  GNU General Public License as published by the Free Software Foundation; 
//  either version 2 of the License, or	(at your option) any later version.
//  but WITHOUT ANY WARRANTY; without even the implied warranty of				
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the				
//  GNU General Public License for more details.						
//  You should have received a copy of the GNU General Public License				
//  along with this program; if not, write to the Free Software					
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.					
////////////////////////////////////////////////////////////////////////////////

#include "maiaKeyboard.h"

/** Call the constructor of the parent class 
    @param p Parent window of all the buttons (may be null)
    @param tick Scan interval in milliseconds
*/
maiaKeyboard::maiaKeyboard(wxWindow *p, int tick)
  : omUIKeyboard()
{
  reachKey = false ;
  parentWindow = p ;
  scanTick = tick ;

  // retrive application options
  currentButton = NULL ;   // default

}

/** Do post constructor initialization */
void maiaKeyboard::Init()
{
  omUIKeyboard::Init() ;
  opts = (maiaOptions *) wxGetApp().objects.options ;
  autoscan = opts->autoscan;
  joystick = opts->joystick;

  // last values
  lastAutoscan = autoscan;
  lastJoystick = joystick;
}

/** Reload options after parameters change */
void maiaKeyboard::Reload()
{
  this->Stop();
  omUIKeyboard::Reload() ;
  scanTick = opts->scanTime;
  autoscan = opts->autoscan;
  joystick = opts->joystick ;

  // if autoscan changes from false to true turn off current button if active
  if ( autoscan && !lastAutoscan )     
    {
      if (currentButton != NULL)
	{
	  ((maiaButton *) currentButton)->ResetCurrent();
	  currentButton == NULL ;
	  RepaintKeyboard();
	}
    }

  // if joystick mode changes from false to ture and autoscan is not actived or
  // autoscan turns from true to false and joystick mod is active
  // then current button must turn on
  if ( (!lastJoystick && joystick && !autoscan) ||
       (lastAutoscan && !autoscan && joystick) )
    {
      this->SetCurrentButton( (maiaButton *) buttons[0]);
      //((maiaButton *) currentButton)->PaintDefaultPicture(); 
      RepaintKeyboard();
    }

  lastAutoscan = autoscan;
  lastJoystick = joystick;

  this->Start();
}


/** This method implement the virtual function of MaiaGenericKeyboard class. 
    The method creates the root node of the tree
*/
void maiaKeyboard::AtStartDocument()
{
  // create a tree root without parent and options and with infinite scan loops
  rootNode = new omUITreeNode( NULL, NULL, -1) ;  
  currentNode = rootNode ;
}


/** This method implement the vitual function of omUIKeyboard class.
    The method activate the keyboard to let possible to catch events and transmit
    them to the root
*/
void maiaKeyboard::AtEndDocument() 
{
  // set the keyboard active
  active = true ;

  if ( !autoscan && joystick) 
    SetCurrentButton( (maiaButton *)buttons[0] ) ;

    
}

/** This method implement the vitual function of MaiaGenericKeyboard class.
    The method parse the tag name to fill correctly the tree.
    @param name tag name
    @param p parameters of the tag
*/
void maiaKeyboard::AtStartTag(wxString name, omXMLParameters p)
{
  omXMLParameters *pars = &p;

  int screenWidth=wxSystemSettings::GetMetric(wxSYS_SCREEN_X);//Get current screen width
  int screenHeight=wxSystemSettings::GetMetric(wxSYS_SCREEN_Y);//Get current screen height

  //Screen width used as referiment in configuration file
  static int screenWidthRef;
  //Screen height used as referiment in configuration file
  static int screenHeightRef;
  //Ratio between referiment display width in configuration file and current display setting.
  static float widthRatio;
  //Ratio between referiment display height in configuration file and current display setting.
  static float heightRatio;

  // ----------------------------------------------------------- KEYBOARD TAG --
  if ( name.CmpNoCase(MAIAXMLKEYBOARDTAG) == 0) {

    if ( opts != NULL )
      opts->keyboardName=pars->GetValue(wxString(MAIAXMLKEYBOARDNAME));
    

    if(!(screenWidthRef=pars->GetIntValue(wxString(MAIAXMLSCREENWIDTH))))
      screenWidthRef=1024;    //Set screen reference or default value
    widthRatio=(float)screenWidth/(float)screenWidthRef;

    
    if(!(screenHeightRef=pars->GetIntValue(wxString(MAIAXMLSCREENHEIGHT))))
      screenHeightRef=768;    //Set screen reference or default value
    heightRatio=(float)screenHeight/(float)screenHeightRef;

  }


  // ----------------------------------------------------------- SCANBLOCK TAG --
  if ( name.CmpNoCase(MAIAXMLSCANBLOCKTAG) == 0 )
    {
      // create a new node and add it to the tree structure
      omUITreeNode *newNode = new omUITreeNode( currentNode, NULL, opts->scanLoopsNumber) ;
      currentNode->AddNode( newNode) ;
      currentNode = newNode ;
      return ;
    }



  // ----------------------------------------------------------- BUTTON TAG --
  if ( name.CmpNoCase(MAIAXMLBUTTONTAG) == 0 )
    {
      reachKey = true ;
      currentKeyData = new maiaButtonData() ;
      return ;
    }

  // ----------------------------------------------------------- BUTTON elements --
  if (reachKey)
    {
      // ----------- images tag
      if ( name.CmpNoCase(MAIAXMLBUTTONIMAGETAG) == 0) 
	{
	  currentKeyData->normalImage = 
	    pars->GetValue(wxString(MAIAXMLBUTTONNORMALIMAGE)) ;
	  currentKeyData->scanImage = 
	    pars->GetValue(wxString(MAIAXMLBUTTONSCANIMAGE)) ;
	  currentKeyData->selImage = 
	    pars->GetValue(wxString(MAIAXMLBUTTONSELIMAGE)) ;
	}


      // ----------- sounds tag
      if ( name.CmpNoCase(MAIAXMLBUTTONSOUNDTAG) == 0) 
	{
	  currentKeyData->scanSound = 
	    pars->GetValue(wxString(MAIAXMLBUTTONSCANSOUND)) ;
	  currentKeyData->selSound = 
	    pars->GetValue(wxString(MAIAXMLBUTTONSELSOUND)) ;
	}


      // -------------- position tag
      if ( name.CmpNoCase(MAIAXMLBUTTONPOSITION) == 0)
	{
	  currentKeyData->x = (int)(widthRatio*pars->GetIntValue(wxString(MAIAXMLBUTTONXPOS))) ;
	  currentKeyData->y = (int)(heightRatio*pars->GetIntValue(wxString(MAIAXMLBUTTONYPOS))) ;	 
	}

      // ----------------- dimension tag
      if ( name.CmpNoCase(MAIAXMLBUTTONDIMENSION) == 0)
	{
	  //+1 save from empty line
	  currentKeyData->h = (int)(heightRatio*pars->GetIntValue(wxString(MAIAXMLBUTTONHEIGHT)))+1;
	  //+1 save from empty line
	  currentKeyData->w = (int)(widthRatio*pars->GetIntValue(wxString(MAIAXMLBUTTONWIDTH)))+1;
	}
      // ------------------------- action tag (not fully implemented)
      if ( name.CmpNoCase(MAIAXMLBUTTONACTIONTAG) == 0 ) 
	{
	  // list of codes must be implemented here
	  currentKeyData->AddAction( new omCommand(pars->GetValue(wxString(MAIAXMLBUTTONACTIONCODE)),
						    pars->GetValue(wxString(MAIAXMLBUTTONACTIONPARS)))) ;
	}

    }

}

/** This method implement the vitual function of MaiaGenericKeyboard class.
    The method parse the tag and if necessary close open jobs (ex insert a button in the
    structure).
    @param name tag name
*/
void maiaKeyboard::AtEndTag( wxString name)
{
  // found a button tag
  // create the button object and insert it into the structure
  if ( name.CmpNoCase(MAIAXMLBUTTONTAG) == 0 )
    {
      // create the new button object with the reader parameters 
      maiaButton *theButton = new maiaButton( parentWindow, true, currentKeyData) ;

      //insert a new value in buttons array
      buttons.insert(buttons.end(), theButton);

      //currentKeyData = NULL ;
      currentNode->AddLeaf(theButton) ;
      reachKey = false ;
      theButton->PaintDefaultPicture() ;
      //theButton->Show() ;
      //theButton->Hide() ;
      
    }

  // found a scanblock tag
  // come back to the parent
  if ( name.CmpNoCase(MAIAXMLSCANBLOCKTAG) == 0)
    {
      currentNode = currentNode->GetParentNode() ;
    }
} 



/** Start automatic scan mode */
void maiaKeyboard::Start()
{
  if (opts->autoscan)
    wxTimer::Start(scanTick) ;
}

/** Stop automatic scan mode */
void maiaKeyboard::Stop()
{
  wxTimer::Stop() ;
}

// ---------------- private methods -------------

/** Implement Notify method of the timer
    pass the tick signal to the root node.
*/
void maiaKeyboard::Notify()
{
  rootNode->Tick() ;
}


/** Link each button with closer upper,lower,left and right button */
void maiaKeyboard::LinkConnectedButtons(){
  //lenght of vector
  int vectorLenght=buttons.size();
  //index to current button
  int currButton;


  //distance value initialized to 0
  float distUpper=0;
  float distLower=0;
  float distLeft=0;
  float distRight=0;

  float currDistance;

  int i;

  //for each button
  for(currButton=0;currButton<vectorLenght;currButton++){

    //distance value initialized to 0
    distUpper=0;
    distLower=0;
    distLeft=0;
    distRight=0;


    //calculate distance with each other button
    for(i=0;i<vectorLenght;i++){

      //button not compared with itself
      if(i!=currButton){
	//Calculate the distance with current button
	currDistance=( ((maiaButton *)buttons[i])->x - ((maiaButton *)buttons[currButton])->x)*(((maiaButton *)buttons[i])->x - ((maiaButton *)buttons[currButton])->x)+
	  (((maiaButton *)buttons[i])->y - ((maiaButton *)buttons[currButton])->y)*(((maiaButton *)buttons[i])->y - ((maiaButton *)buttons[currButton])->y);
	


	//if UPPER
	if(( ((maiaButton *)buttons[currButton])->y - ((maiaButton *)buttons[i])->y)>0 ){
	  //if distUpper not setted than distUpper=currDistance
	  if(distUpper==0){
	    distUpper=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->upper=buttons[i];
	  }
	  //if currDistance<distUpper than distUpper=currDistance
	  if(currDistance<distUpper){

	    distUpper=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->upper=buttons[i];
	  }
	}





	//if LOWER
	if( (((maiaButton *)buttons[currButton])->y - ((maiaButton *)buttons[i])->y)<0 ){
	  //if distLower not setted than distLower=currDistance
	  if(distLower==0){
	    distLower=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->lower=buttons[i];
	  }
	  //if currDistance<distUpper than distUpper=currDistance
	  if(currDistance<distLower){
	    distLower=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->lower=buttons[i];
	  }
	}




	//if RIGHT
	if( (((maiaButton *)buttons[i])->x - ((maiaButton *)buttons[currButton])->x)>0 ){
	  //if distRight not setted than distRight=currDistance
	  if(distRight==0){
	    distRight=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->right=buttons[i];
	  }
	  //if currDistance<distRight than distRight=currDistance
	  if(currDistance<distRight){
	    distRight=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->right=buttons[i];
	  }
	}




	//if LEFT
	if( (((maiaButton *)buttons[i])->x - ((maiaButton *)buttons[currButton])->x)<0 ){
	  //if distLeft not setted than distRight=currDistance
	  if(distLeft==0){
	    distLeft=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->left=buttons[i];
	  }
	  //if currDistance<distLeft than distLeft=currDistance
	  if(currDistance<distLeft){ 
	    distLeft=currDistance;
	    //Link current button with 'i' button
	    ((maiaButton *)buttons[currButton])->left=buttons[i];
	  }
	}



      }//end if

    }//end for

  }//end for


}


/** Set the current button.
    Used in joystick mode navigation.

    @param current pointer to current button
*/
void maiaKeyboard::SetCurrentButton( omObject *current)
{
  if ( currentButton != NULL) 
    ((maiaButton *)currentButton)->ResetCurrent() ;

  currentButton = current ;
  ((maiaButton *) currentButton)->SetCurrent() ;
}



/** Get the current button.

    @return pointer to current button or NULL
*/
omObject *maiaKeyboard::GetCurrentButton()
{
  return currentButton ;
}
  

/** Process Action event. */
void maiaKeyboard::Action()
{
  if (!opts->autoscan && opts->joystick)
    {
      ((maiaButton *)currentButton)->Execute() ;
      return ;
    }

  // pass the signal to the parent method
  omUIKeyboard::Action() ;

}

/** Process GoDown event. */
void maiaKeyboard::GoDown()
{
  if (!opts->autoscan && opts->joystick)
    {
      maiaButton *cur = (maiaButton *)GetCurrentButton() ;
      omObject *next = cur->lower ;
      if ( next != NULL) 
	{
	  SetCurrentButton(next) ;
	  cur->Clean() ;
	  ((maiaButton *)next)->Clean() ;
	  wxGetApp().Yield() ;
	}
    }
}

/** Process GoUp event. */
void maiaKeyboard::GoUp()
{
  if (!opts->autoscan && opts->joystick)
    {
      maiaButton *cur = (maiaButton *)GetCurrentButton() ;
      omObject *next = cur->upper ;
      if ( next != NULL) 
	{
	  SetCurrentButton(next) ;
	  cur->Clean() ;
	  ((maiaButton *)next)->Clean() ;
	  wxGetApp().Yield() ;
	}
    }
}

/** Process GoLeft event. */
void maiaKeyboard::GoLeft()
{
  if (!opts->autoscan && opts->joystick)
    {
      maiaButton *cur = (maiaButton *)GetCurrentButton() ;
      omObject *next = cur->left ;
      if ( next != NULL) 
	{
	  SetCurrentButton(next) ;
	  cur->Clean() ;
	  ((maiaButton *)next)->Clean() ;
	  wxGetApp().Yield() ;
	}
    }
}

/** Process GoRight event. */
void maiaKeyboard::GoRight()
{
  if (!opts->autoscan && opts->joystick)
    {
      maiaButton *cur = (maiaButton *)GetCurrentButton() ;
      omObject *next = cur->right ;
      if ( next != NULL) 
	{
	  SetCurrentButton(next) ;
	  cur->Clean() ;
	  ((maiaButton *)next)->Clean() ;
	  wxGetApp().Yield() ;
	}
    }
}


/** Load the keyboard using MaiaGenericKeyboard function */
void maiaKeyboard::LoadKeyboard(wxString keyb)
{
  buttons.clear();//Empty the buttons vector

  chdir("keyboard");

  omUIKeyboard::LoadKeyboard(keyb) ;

  // check if keyboard grab is needed
  //MaiaEventCheck *eventChecker = (MaiaEventCheck *) wxGetApp().objects.eventChecker ;

//   if (opts->grabKeys)
//     if (eventChecker != NULL)
//       eventChecker->GrabActionKeys() ;

  //Link each button with next button for joystick navigation
  //LinkConnectedButtons();
  omUIFunctions::Create2DButtonMap( buttons) ;
  chdir("..") ;
}
